<?php

namespace TC\PaygateRuKassa\Payment;

use Curl\Curl;
use XF;
use XF\Entity\PaymentProfile;
use XF\Entity\PurchaseRequest;
use XF\Http\Request;
use XF\Mvc\Controller;
use XF\Payment\AbstractProvider;
use XF\Payment\CallbackState;
use XF\Purchasable\Purchase;
use XF\Repository\Payment;

class RuKassa extends AbstractProvider
{
	/**
	 * @return string
	 */
	public function getTitle()
	{
		return 'RuKassa';
	}

	/**
	 * @param array $options
	 * @param array $errors
	 *
	 * @return bool
	 */
	public function verifyConfig(array &$options, &$errors = [])
	{
		if (empty($options['shop_id']) || empty($options['shop_token']))
		{
			$errors[] = XF::phrase('tc_RuKassa_you_must_provide_all_data');
		}

		if (!$errors)
		{
			return true;
		}

		return false;
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $amount
	 *
	 * @return float
	 */
	private function getCostWithTax(PaymentProfile $paymentProfile, $amount)
	{
		$paymentOptions = $paymentProfile->options;

		if (!empty($paymentOptions['tax']))
		{
			$tax = str_replace(',', '.', $paymentOptions['tax']);

			$amount += $amount * (floatval($tax) / 100);
		}

		return round($amount, 2);
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase        $purchase
	 *
	 * @return string
	 */
	protected function getReturnUrl(PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		return $purchase->returnUrl;
	}

	/**
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase        $purchase
	 *
	 * @return array
	 */
	public function getPaymentParams(PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;
		$purchaser = $purchase->purchaser;

		$purchase->cost = $this->getCostWithTax($paymentProfile, $purchase->cost);
	
		$data = [
			'email'		   => $purchase->extraData['email'] ?? $purchaser->email,
			'account'  	   => $purchaser->user_id,
			'comment'      => $purchase->title,
			'requestKey'   => $purchaseRequest->request_key,
		];

		$params = [
			'shop_id'      => $paymentProfile->options['shop_id'],
			'token'        => $paymentProfile->options['shop_token'],
			'order_id'	   => time() . mt_rand(),
			'amount'       => $purchase->cost,
			'currency'	   => $purchase->currency,
			'data'		   => json_encode($data)
		];

		return $params;
	}

	/**
	 * @param Controller      $controller
	 * @param PurchaseRequest $purchaseRequest
	 * @param Purchase        $purchase
	 *
	 * @return XF\Mvc\Reply\Error|XF\Mvc\Reply\Redirect|XF\Mvc\Reply\View
	 */
	public function initiatePayment(Controller $controller, PurchaseRequest $purchaseRequest, Purchase $purchase)
	{
		$paymentProfile = $purchase->paymentProfile;

		if (empty($paymentProfile->options['shop_id']) || empty($paymentProfile->options['shop_token']))
		{
			return $controller->noPermission();
		}

		$params = $this->getPaymentParams($purchaseRequest, $purchase);

		$viewParams = [
			'paymentProfile'  => $paymentProfile,
			'purchaseRequest' => $purchaseRequest,
			'purchase'        => $purchase,
			'params'          => $params
		];

		/*if(!empty($paymentProfile->options['widget']))
		{
			return $controller->view('TC\RuKassa:Purchase\Initiate', 'tc_payment_initiate_RuKassa', $viewParams);
		}*/

		return $controller->redirect($this->createPaymentForm($params));
	}

	/**
	 * @param Request $request
	 *
	 * @return CallbackState
	 */
	public function setupCallback(Request $request)
	{
		$state = new CallbackState();

		$state->input = $_POST;

		$state->input['params'] = json_decode($state->input['data'], true);


		$state->transactionId = $state->input['id'];
		$state->subscriberId = $state->input['params']['email'];
		$state->requestKey = $state->input['params']['requestKey'];

		$state->costAmount = $state->input['amount'];
		$state->costCurrency = 'RUB';

		$state->signature = $request->getServer('HTTP_SIGNATURE');
		$state->input['signature'] = $state->signature;

		$state->ip = $request->getIp();

		$state->httpCode = 200;

		return $state;
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validateTransaction(CallbackState $state)
	{
		if (!$state->transactionId && !$state->requestKey)
		{
			$state->logType = 'info';
			$state->logMessage = 'No transaction ID or signature. No action to take.';

			return false;
		}

		/** @var Payment $paymentRepo */
		$paymentRepo = XF::repository('XF:Payment');
		if ($paymentRepo->findLogsByTransactionId($state->transactionId)->total())
		{
			// Transaction already processed. Skipping.
			$state->logType = null;

			return false;
		}

		return true;
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validatePurchasableData(CallbackState $state)
	{
		$paymentProfile = $state->getPaymentProfile();

		$options = $paymentProfile->options;
		if (empty($options['shop_token']))
		{
			$state->logType = 'error';
			$state->logMessage = 'Invalid shop_token.';

			return false;
		}

		if (!empty($state->signature))
		{
			if($state->signature == hash_hmac('sha256', $state->input['id'] . '|' . $state->input['createdDateTime'] . '|' . $state->input['amount'], $paymentProfile->options['shop_token']))
			{
				return true;
			}
		}
		
		if ($state->input['in_amount'] < $state->input['amount'])
		{
			$state->logType = 'error';
			$state->logMessage = 'Not enough amount';

			return false;
		}

		$state->logType = 'error';
		$state->logMessage = 'Invalid signature.';

		return false;
	}

	/**
	 * @param CallbackState $state
	 *
	 * @return bool
	 */
	public function validateCost(CallbackState $state)
	{

		$purchaseRequest = $state->getPurchaseRequest();
		$cost = $this->getCostWithTax($state->paymentProfile, $purchaseRequest->cost_amount);

		$costValidated = (round($state->costAmount, 2) == round($cost, 2)
			&& $this->verifyCurrency($state->paymentProfile, $state->costCurrency)
		);

		if ($costValidated)
		{
			return true;
		}

		$state->logType = 'error';
		$state->logMessage = 'Invalid cost amount';

		return false;
	}

	/**
	 * @param CallbackState $state
	 */
	public function getPaymentResult(CallbackState $state)
	{
		if ($state->input['status'] == "PAID")
		{
			$state->paymentResult = CallbackState::PAYMENT_RECEIVED;
		}
	}

	/**
	 * @param CallbackState $state
	 */
	public function prepareLogData(CallbackState $state)
	{
		$state->logDetails = $state->input ?: [];
	}

	/**
	 * @var array
	 */
	protected $supportedCurrencies = [
		"RUB",
		"USD"
	];

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $currencyCode
	 *
	 * @return bool
	 */
	public function verifyCurrency(PaymentProfile $paymentProfile, $currencyCode)
	{
		return (in_array($currencyCode, $this->supportedCurrencies));
	}

	/**
	 * @param PaymentProfile $paymentProfile
	 * @param                $unit
	 * @param                $amount
	 * @param int            $result
	 *
	 * @return bool
	 */
	public function supportsRecurring(PaymentProfile $paymentProfile, $unit, $amount, &$result = self::ERR_NO_RECURRING)
	{
		$result = self::ERR_NO_RECURRING;

		return false;
	}

	/**
	 * @param CallbackState $state
	 */
	public function completeTransaction(CallbackState $state)
	{
		parent::completeTransaction($state);

		if ($state->paymentResult == CallbackState::PAYMENT_RECEIVED)
		{
			$state->logMessage = json_encode(['error' => '0']);
		}
	}
	
	/**
	 * @param array $params
	 *
	 * @return URL
	 */
	protected function createPaymentForm($params)
	{
		$curl = new Curl();
		$curl->post('https://lk.rukassa.is/api/v1/create', $params);
		$json = json_decode($curl->response);
		return $json->url;
	}
}